<?php

namespace App\Http\Controllers\Member\Controller;

use App\Exceptions\ApiForbidden;
use App\Exceptions\ApiTimeout;
use App\Http\Controllers\ControllerBase;
use App\Lib\Response\HttpStatus;
use App\Lib\Response\Response;
use App\Exceptions\ApiUnauthorized;
use App\Exceptions\ApiValidationFailed;
use Illuminate\Routing\Controller;
use Illuminate\Http\Request;

class User extends ControllerBase
{
    /**
     * 获取一个登录二维码
     * @method get
     * @param $code optional
     * router /user/code/code\S*
     */
    public function getLoginImg($code = null)
    {
        try {
            $client = app("client");
            $client->setUrl("user/auth/code/".$code);
            $client->setMethod("get");
            $res = $client->curl();
            $qrcode = app("qrcode");
            $qrcode->setText($res);
            $img = $qrcode->getImage();
            return Response::success($this->getImage($img),$res);
        } catch (\Exception $e){
            return Response::error($e->getMessage());
        }
    }

    /**
     * 获取登录状态
     * @method get
     * @param $code required
     * @router /user/status/code\S+
     */
    public function getLoginStatus($code)
    {
        try {
            $client = app("client");
            $client->setUrl("user/show/" . $code);
            $client->setMethod("get");
            $res = $client->curl();
            return Response::success($res);
        } catch (ApiUnauthorized $e) {
            return Response::error($e->getMessage(), HttpStatus::HttpUnauthorized);
        } catch (ApiTimeout $e) {
            return Response::error($e->getMessage(), HttpStatus::HttpTimeout);
        } catch (\Exception $e) {
            return Response::error($e->getMessage());
        }
    }

    protected function getImage($resource)
    {
        $stream = null;
        ob_start();
        header("Content-type: image/png");
        imagepng($resource);
        $stream = base64_encode(ob_get_contents());
        ob_end_clean();
        return $stream;
    }

    /**
     * 申请成为摄影师
     * @method get
     * @param $code required
     * @router /user/apply
     */
    public function Apply(Request $request)
    {
        try {
            $uid = $this->getAuth();
            $client = app("client");
            $client->setUrl("ident/" . $uid);
            $client->setMethod("post");
            $client->setData($request->all());
            $res = $client->curl();
            return Response::success($res);
        } catch (ApiUnauthorized $e) {
            return Response::error($e->getMessage(),HttpStatus::HttpUnauthorized);
        } catch (ApiForbidden $e) {
            return Response::error("您已申请摄影师",HttpStatus::HttpForbidden);
        } catch (\Exception $e) {
            return Response::error($e->getMessage());
        }
    }

    /**
     * 获取第三方OPENID
     */
    public function getOAuth($platform = "") {
        try {
            empty($platform) && $platform = "wx";
            $this->prefix = "user";
            $res = parent::show($this->getAuth(),$this->combine(['relation' => 'third']));
            $data = $this->filter($res['data']->relation_third,function($item) use ($platform){
                if ($item->platform == $platform) {
                    return $item->token;
                }
            });
            if (count($data) > 0) {
                $data = $data[0];
            }
            return Response::success($data->token);
        } catch (ApiUnauthorized $e) {
            return Response::error($e->getMessage(),HttpStatus::HttpUnauthorized);
        } catch (\Exception $e) {
            return Response::error($e->getMessage());
        }
    }

}