<?php

namespace App\Lib\Client;

use App\Lib\Response\HttpStatus;

class CurlClient
{
    protected $error;
    protected $method = "GET";
    protected $config;
    protected $data;
    protected $url;
    private $format = "JSON";
    protected $header = [];
    protected $base;

    const Exceptions = [
        HttpStatus::HttpNotFound => \App\Exceptions\ApiNotFound::class,
        HttpStatus::HttpUnauthorized => \App\Exceptions\ApiUnauthorized::class,
        HttpStatus::HttpValidationFailed => \App\Exceptions\ApiValidationFailed::class,
        HttpStatus::HttpForbidden  => \App\Exceptions\ApiForbidden::class,
        HttpStatus::HttpInvalidArgument => \Exception::class,
        HttpStatus::HttpTimeout => \App\Exceptions\ApiTimeout::class,
    ];

    public function __construct($header = [])
    {
        $this->base = "http://api.yuepai.com/";
    }

    public function setUrl($url)
    {
        $this->url = $url;
    }

    public function getUrl()
    {
       return $this->url;
    }

    /**
     * @param mixed $data
     */
    public function setData($data)
    {
        $this->data = $data;
    }

    /**
     * @param string $method
     */
    public function setMethod($method)
    {
        $this->method = strtoupper($method);
    }

    /**
     * @param array $header
     */
    public function setHeader($header)
    {
        $this->header = array($this->header,$header);
    }


    public function curl()
    {
        $http = curl_init();
        curl_setopt($http,CURLOPT_RETURNTRANSFER,true);
        curl_setopt($http,CURLOPT_URL,$this->base.$this->url);
        curl_setopt($http,CURLOPT_HEADER,false);
        curl_setopt($http,CURLOPT_CUSTOMREQUEST,$this->method);
        if ($this->format == "JSON")
        {
            $this->header = array_merge($this->header,['Content-Type:application/json']);
            $this->data = json_encode($this->data);
        }
        if ($this->method === "POST") {
            curl_setopt($http,CURLOPT_POST,true);
        }
        curl_setopt($http,CURLOPT_HTTPHEADER,$this->header);
        curl_setopt($http,CURLOPT_POSTFIELDS,$this->data);
        $res = curl_exec($http);
        $curlinfo = curl_getinfo($http);
        if ($curlinfo == false || (is_array($curlinfo) && $curlinfo['http_code'] != 200)) {
            var_dump($res);
            throw new \Exception("返回错误");
        }
        $res = json_decode($res);
        if (json_last_error() > 0) {
            throw new \Exception(json_last_error_msg());
        }
        if ($res->code >= 200 && $res->code < 400) {
            if ($this->method == "POST" || $this->method == "PUT") {
                return "";
            } else {
                return $res->data;
            }
        } else {
            $exception = self::Exceptions[$res->code];
            $msg = is_array($res->message)?implode(";",$res->message):$res->message;
            throw new $exception($msg);
        }

    }

    public function getError()
    {
        return $this->error;
    }
}